######### KD tree node data structure
mutable struct Node
    key::Vector{Int}
    father::Union{Node, Nothing}
    info::Vector{Int}
    left::Union{Node, Nothing}
    right::Union{Node, Nothing}

    function Node(key, father, info)
        new(copy(key), father, copy(info), nothing, nothing)
    end
end

const K = 3                               # Define the dimension of the KD tree

####################### Add a new node in a KD tree ###########################
function kd_tree_add(root::Union{Node, Nothing},        # Root of a (sub-) tree
                     key,                             # Key for splitting nodes
                     info,                   # Information to store in the node
                     depth)                                 # Depth of the root
    if isnothing(root)
        return Node(key, nothing, info)
    elseif root.key[mod1(depth, K)] < key[mod1(depth, K)]
        if isnothing(root.right)
            root.right = Node(key, root, info)
        else
            root.right = kd_tree_add(root.right, key, info, depth + 1)
        end
    else
        if isnothing(root.left)
            root.left = Node(key, root, info)
        else
            root.left = kd_tree_add(root.left, key, info, depth + 1)
        end
    end
    return root
end


############# Scan a KD tree and print key and info for each node #############
function kd_tree_scan(root::Union{Node, Nothing})
    if !isnothing(root)
        if !isnothing(root.left)
            kd_tree_scan(root.left)
        end
        println("Key: ", root.key, " Info: ", root.info)
        if !isnothing(root.right)
            kd_tree_scan(root.right)
        end
    end
end
