########## Find the node with min or max value in a given dimension ###########
function kd_tree_find_opt(root::Union{Node, Nothing}, # Root of a KD (sub-)tree
                          dim,       # Dimension in which optimum is looked for
                          depth,                            # Depth of the root
                          minimum, # Look for minimum (True) or maximum (False)
                          value,                     # Best value already known
                          opt)                        # Node with optimum value
    depth_opt = -1
    if (minimum && (value > root.key[dim])) || 
       ((!minimum) && (value < root.key[dim]))
        opt = root
        value = root.key[dim]
        depth_opt = depth
    end
    if !isnothing(root.left)
        opt, value, depth_opt = kd_tree_find_opt(root.left, dim, depth + 1, 
                                                 minimum, value, opt)
    end
    if !isnothing(root.right)
        opt, value, depth_opt = kd_tree_find_opt(root.right, dim, depth + 1, 
                                                 minimum, value, opt)
    end
    return opt, value, depth_opt
end

################### Delete the root of a KD (sub-) tree #######################
function kd_tree_delete(root::Union{Node, Nothing},            # Node to delete
                        depth)                              # Depth of the node
    if !isnothing(root.left)
        replacing, val_repl, depth_repl = kd_tree_find_opt(root.left, 
                               mod1(depth, K), depth + 1, false, -Inf, nothing)
    elseif !isnothing(root.right)
        replacing, val_repl, depth_repl = kd_tree_find_opt(root.right, 
                                 mod1(depth, K), depth + 1, true, Inf, nothing)
    else
        if !isnothing(root.father)
            if root.father.left == root
                root.father.left = nothing
            else
                root.father.right = nothing
            end
        end
        return nothing
    end

    root.key = copy(replacing.key)
    root.info = copy(replacing.info)
    kd_tree_delete(replacing, depth_repl)
    return root
end
