########## Tell if node is in the box bounded by minimum and maximum ##########
function kd_tree_in(node, 
                    minimum, maximum)       # Lower and upper corner of the box
    i, result = 1, true
    while i <= K && result
        result = minimum[i] <= node.key[i] <= maximum[i]
        i += 1
    end
    return result
end

### Find a node (if any) with its depth in the box bounded by mini and maxi ###
function kd_tree_find(root,  # Root of the tree in which the node is looked for
                      mini, maxi,           # Lower and upper corner of the box
                      depth)                                # Depth of the root
    if isnothing(root) return nothing, -1 end
    if kd_tree_in(root, mini, maxi) return root, depth end

    if maxi[mod1(depth, K)] >= root.key[mod1(depth, K)]
        result, depth_found = kd_tree_find(root.right, mini, maxi, depth + 1)
        if !isnothing(result) return result, depth_found end
    end
    if mini[mod1(depth, K)] <= root.key[mod1(depth, K)]
        result, depth_found = kd_tree_find(root.left, mini, maxi, depth + 1)
        if !isnothing(result) return result, depth_found end
    end
    return nothing, -1
end

include("kd_tree_delete.jl")                                        # Code %*\ref{lst:kd_tree_delete}*)

########## Remove points of Pareto front dominated by costs (if any) ##########
function update_3opt_pareto(pareto,            # Current Pareto front to update
                            costs,           # New point to be eventually added
                            successors, distances)  # Problem solution and data
    minimum = [-Inf for _ in 1:K]
    maximum = [Inf for _ in 1:K]
    dominant, depth = kd_tree_find(pareto, minimum, costs, 1)
    if isnothing(dominant)
        while true                         # No point of pareto dominates costs
            dominated, depth = kd_tree_find(pareto, costs, maximum, 1)
            if isnothing(dominated)              # All dominated points removed
              break 
            end 
            if dominated == pareto
                pareto = kd_tree_delete(dominated, depth)
            else
                kd_tree_delete(dominated, depth)
            end
        end
        pareto = kd_tree_add(pareto, costs, successors, 1)
        pareto = tsp_3opt_pareto(pareto, costs, successors, distances)
    end
    return pareto
end
