"""
Program to test various local improvement methods
Example of execution:
Number of cities:
500
Random solution: 24565
Cost of solution found with 2-opt first 953
Solution improved with 3-opt limited (100 cities) 870
Solution improved with complete 3-opt 672
Solution improved with 2-opt best 669
"""

include("random_generators.jl")                                     # Code %*\ref{lst:random_generators}*)
include("tsp_utilities.jl")                                         # Code %*\ref{lst:tsp_utilities}*)
include("tsp_2opt_first.jl")                                        # Code %* \ref{lst:tsp_2opt_first}*)
include("tsp_2opt_best.jl")                                         # Code %* \ref{lst:tsp_2opt_best}*)
include("tsp_3opt_limited.jl")                                      # Code %* \ref{lst:tsp_3opt_limited}*)
include("tsp_3opt_first.jl")                                        # Code %* \ref{lst:tsp_3opt_first}*)


function main()
    println("Number of cities:")
    n = parse(Int, readline())
    
    distance = rand_sym_matrix(n, 1, 99)
    solution = rand_permutation(n)
    tour_length = tsp_length(distance, solution)
    println("Random solution: $tour_length")
    
    solution, tour_length = tsp_2opt_first(distance, solution, tour_length)
    println("Cost of solution found with 2-opt first $tour_length")
    
    succ = tsp_tour_to_succ(solution)
    succ, tour_length = tsp_3opt_limited(distance, 100, succ, tour_length)
    println("Solution improved with 3-opt limited (100 cities) $tour_length")
    
    succ, tour_length = tsp_3opt_first(distance, succ, tour_length)
    println("Solution improved with complete 3-opt $tour_length")
    
    solution = tsp_succ_to_tour(succ)
    solution, tour_length = tsp_2opt_best(distance, solution, tour_length)
    println("Solution improved with 2-opt best $tour_length")
end

main()

