"""
Program to test pareto local seach for the TSP with 3-opt moves
Example of run with K = 3 (KD-tree Key = costs; Info = tour)

Number of cities: 
6
Key: [137, 253, 273] Info: [6, 4, 1, 5, 3, 2]
Key: [173, 287, 236] Info: [3, 5, 6, 1, 4, 2]
Key: [222, 288, 235] Info: [3, 4, 5, 1, 6, 2]
Key: [263, 249, 242] Info: [4, 6, 5, 2, 1, 3]
Key: [172, 265, 244] Info: [5, 6, 1, 2, 4, 3]
Key: [182, 224, 320] Info: [6, 3, 5, 1, 4, 2]
Key: [184, 244, 297] Info: [2, 6, 5, 1, 4, 3]
Key: [166, 340, 264] Info: [2, 4, 1, 5, 6, 3]
Key: [246, 367, 201] Info: [4, 5, 1, 2, 6, 3]
"""

include("random_generators.jl")                                     # Code %*\ref{lst:random_generators}*)
include("kd_tree_add_scan.jl")                                      # Code %*\ref{lst:kd_tree_add_scan}*)
include("tsp_3opt_pareto.jl")                                       # Code %*\ref{ lst:tsp_3opt_pareto}*)

println("Number of cities: ")
n = parse(Int, readline())

distance = [rand_sym_matrix(n, 1, 99) for _ in 1:K]
successors = [mod1(i + 1, n) for i in 1:n]
costs = zeros(Int, K)
for dim in 1:K
    for i in 1:n
        costs[dim] += distance[dim][i,successors[i]]
    end
end

pareto = tsp_3opt_pareto(nothing, costs, successors, distance)

kd_tree_scan(pareto) # Print pareto front with tours (successors representation)

