"""
Program to test a Taboo Search for the TSP
Example of run:
Number of cities: 
30
Number of tabu iterations: 
200
Minimum tabu_duration: 
4
Maximum tabu_duration: 
20
Penalty: 
0.005
TS 1 1190
TS 2 1053
TS 3 906
TS 4 796
...
TS 29 177
TS 46 174
TS 119 173
Cost of solution found : 173
[14, 1, 7, 6, 28, 29, 2, 5, 30, 21, 3, 8, 17, ...25, 10, 22, 26, 11, 23, 20, 15]
"""

include("random_generators.jl")                                     # Code %*\ref{lst:random_generators}*)
include("tsp_utilities.jl")                                         # Code %*\ref{lst:tsp_utilities}*)
include("tsp_TS.jl")                                                # Code %* \ref{lst:tsp_TS}*)

println("Number of cities: ")
n = parse(Int, readline())

distances = rand_sym_matrix(n, 1, 99)
solution = rand_permutation(n)
tour_length = tsp_length(distances, solution)

println("Number of tabu iterations: ")
iterations = parse(Int, readline())
println("Minimum tabu duration: ")
min_tabu = parse(Int, readline())
println("Maximum tabu duration: ")
max_tabu = parse(Int, readline())
println("Penalty: ")
freq_penalty = parse(Float64, readline())

solution, tour_length = tsp_TS(distances, solution, tour_length, iterations, 
                               min_tabu, max_tabu, freq_penalty)
println("Cost of solution found: $tour_length")
println(solution)

