# Local search based on 3-opt moves, limited to a subproblem of size r
function tsp_3opt_limited(d, r, succ, tour_length)
    n = length(succ)
    r = r > n ? n : r              # Subpath cannot be larger than a whole tour
    i = last_i = 1                       # First subpath starts from city i = 1
    while true                         
        j = succ[i]
        edges_ij, edges_jk = 1, 1 # Number of edges from i to j and from j to k
        while edges_ij + edges_jk < r           # Do not exceed subproblem size
            k = succ[j]
            while edges_ij + edges_jk < r 
                delta = d[i, succ[j]] + d[j, succ[k]] + d[k, succ[i]] -
                        d[i, succ[i]] - d[j, succ[j]] - d[k, succ[k]]
                if delta < 0
                    tour_length += delta
                    succ[i], succ[j], succ[k] = succ[j], succ[k], succ[i]
                    j, k  = k, j                    # j must be between i and k
                    edges_ij, edges_jk = edges_jk, edges_ij 
                    last_i = i
                end
                edges_jk += 1
                k = succ[k]
            end
            edges_jk = 1
            edges_ij += 1
            j = succ[j]
        end
        i = succ[i]
        if i == last_i
            break
        end
    end
    return succ, tour_length
end
