include("kd_tree_update_pareto.jl")                                 # Code %*\ref{lst:kd_tree_update_pareto}*)

######### Pareto local search for the TSP based on 3-opt neighbourhood
function tsp_3opt_pareto(pareto,                                 # Pareto front
                         costs,              # Tour length (for each dimension)
                         s,                 # Solution (successor of each city)
                         d)          # Distance matrix (one for each dimension)
    costs_neighbour = fill(-1, K)                  # Cost of neighbour solution
    start = unif(1,length(s))        # Random starting city for move evaluation
    i, j, k = start, s[start], s[s[start]]             # Indices of a 3opt move
    while s[s[i]] != start             # Neighbourhood not completely evaluated
        for dim in 1:K
            costs_neighbour[dim] = 
                costs[dim] + d[dim][i,s[j]] + d[dim][j,s[k]] + d[dim][k,s[i]] - 
                             d[dim][i,s[i]] - d[dim][j,s[j]] - d[dim][k,s[k]]
        end
        s[i], s[j], s[k] = s[j], s[k], s[i]      # Change solution to neighbour
        pareto = update_3opt_pareto(pareto, costs_neighbour, s, d)
        s[k], s[j], s[i] = s[j], s[i], s[k]                  # Back to solution
        k = s[k]                                                       # Next k
        if k == i                                 # k at its last value, next j
            j = s[j]; k = s[j]
        end                 
        if k == i                                 # j at its last value, next i
            i = s[i]; j = s[i]; k = s[j] 
        end       
    end
    return pareto
end
