include("rank_based_selection.jl")                                  # Code %*\ref{lst:rank_based_selection}*)
include("OX_crossover.jl")                                          # Code %*\ref{lst:OX_crossover}*)
include("mutate.jl")                                                # Code %*\ref{lst:mutate}*)
include("insert_child.jl")                                          # Code %*\ref{lst:insert_child}*)
include("tsp_LK.jl")                                                # Code %*\ref{lst:tsp_LK}*)

######### Basic Memetic Algorithm for the TSP
function tsp_GA(d,                      # Distance matrix (must be symmetrical)
                population_size,                       # Size of the population
                generations,                            # Number of generations
                mutation_rate)
    n = size(d, 1)
    population = [rand_permutation(n) for _ in 1:population_size]
    lengths = [tsp_length(d, population[i]) for i in 1:population_size]

    order = collect(1:population_size)   # Population sorted by increasing cost
    for i in 1:(population_size - 1)
        for j in (i + 1):population_size
            if lengths[order[i]] > lengths[order[j]]
                order[i], order[j] = order[j], order[i]
            end
        end
    end
    println("GA initial best individual $(lengths[order[1]])")

    for gen in 1:generations
        parent1 = rank_based_selection(population_size)
        parent2 = rank_based_selection(population_size)
        child = OX_crossover(population[order[parent1]], 
                             population[order[parent2]])
        child = mutate(mutation_rate, child)
        child_length = tsp_length(d, child)
        child, child_length = tsp_LK(d, child, child_length)
        child_rank, population, lengths, order = 
            insert_child(child, child_length, 
                         population_size, population, lengths, order)
        if child_rank == 1
            println("GA improved tour $(gen) $(child_length)")
        end
    end
    return population[order[1]], lengths[order[1]]
end
