include("random_generators.jl")                                     # Code %*\ref{lst:random_generators}*)
include("tsp_utilities.jl")                                         # Code %*\ref{lst:tsp_utilities}*)
include("tsp_LK.jl")                                                # Code %*\ref{lst:tsp_LK}*)

######### Procedure for producing one TSP tour using GRASP principles
function tsp_GRASP(d,                                         # Distance matrix
                   alpha)
    n = size(d, 1)
    tour = rand_permutation(n)
    for i in 1:(n - 1)
        # determine c_min and c_max incremental costs
        c_min, c_max = Inf, -Inf
        for j in (i + 1):n
            if c_min > d[tour[i], tour[j]]
                c_min = d[tour[i], tour[j]]
            end
            if c_max < d[tour[i], tour[j]]
                c_max = d[tour[i], tour[j]]
            end
        end
        next = i + 1        # Find the next city to insert, based on lower cost
        while d[tour[i], tour[next]] > c_min + alpha * (c_max - c_min)
            next += 1
        end
        tour[i + 1], tour[next] = tour[next], tour[i + 1]
    end
    tour_length = tsp_length(d, tour)
    return tsp_LK(d, tour, tour_length)
end
