include("tsp_GRASP.jl")                                             # Code %*\ref{lst:tsp_GRASP}*)
include("tsp_path_relinking.jl")                                    # Code %*\ref{lst:tsp_path_relinking}*)

######### GRASP with path relinking for the TSP
function tsp_GRASP_PR(d, # Distance matrix
                      iterations,                    # Number of calls to GRASP
                      population_size,                 # Size of the population
                      alpha)                                  # GRASP parameter
    n = size(d, 1)
    population = fill([-1], population_size)
    pop_size = iteration = 0
    lengths = fill(-1, population_size)
    while pop_size < population_size && iteration < iterations
        tour, tour_length = tsp_GRASP(d, alpha)
        iteration += 1
        succ = tsp_tour_to_succ(tour)
        different = true
        for i in 1:pop_size
            if tsp_compare(population[i], succ) == 0
                different = false
                break                       # The tour is already in population
            end
        end
        if different
            pop_size += 1
            population[pop_size], lengths[pop_size] = copy(succ), tour_length
         end
    end
    if iteration == iterations  # Unable to generate enough different solutions
        population_size = pop_size
    end
    for it in iteration:iterations
        tour, tour_length = tsp_GRASP(d, alpha)
        iteration += 1
        succ = tsp_tour_to_succ(tour)
        successors, tour_length = 
            tsp_path_relinking(d, population[unif(1,population_size)], 
                               tour_length, succ)
        max_difference, replacing = -1, -1
        for i in 1:population_size
            if tour_length <= lengths[i]
                difference = tsp_compare(population[i], successors)
                if difference == 0
                    max_difference = 0
                    break
                end
                if difference > max_difference && tour_length < lengths[i]
                    max_difference = difference
                    replacing = i
                end
            end
        end
        if max_difference > 0
            lengths[replacing] = tour_length
            population[replacing] = copy(successors)
            println("GRASP_PR population updated: $it $tour_length")
        end
    end
    best = 1
    for i in 2:population_size
        if lengths[i] < lengths[best]
            best = i
        end
    end
    return tsp_succ_to_tour(population[best]), lengths[best]
end
