include("tsp_utilities.jl")                                         # Code %*\ref{lst:tsp_utilities}*)
######### Basic Lin & Kernighan improvement procedure for the TSP
function tsp_LK(D,                                            # Distance matrix
                tour,                                                # Solution
                cost)                                             # Tour length
    n = length(tour)
    succ = tsp_tour_to_succ(tour)
    for i in 1:n
        succ[tour[i]] = tour[mod(i, n) + 1]
    end
    tabu = [zeros(Int, n) for _ in 1:n]              # Can edge i-j be removed?
    iteration = 0           # Outermost loop counter to identify tabu condition
    last_a, a = 1, 1                  # Initiate ejection chain from city a = 1
    improved = true
    while a != last_a || improved
        improved = false
        iteration += 1
        b = succ[a]
        path_length = cost - D[a, b]
        path_modified = true
        while path_modified  # Identify best ref. struct. with edge a-b removed
            path_modified = false
            ref_struct_cost = cost       # Cost of reference structure retained
            c = best_c = succ[b]
            while succ[c] != a                     # Ejection can be propagated
                d = succ[c]
                if path_length - D[c, d] + D[c, a] + D[b, d] < cost
                    best_c = c            # An improving solution is identified
                    ref_struct_cost = path_length - D[c, d] + D[c, a] + D[b, d]
                    break               # Change improving solution immediately
                end
                if tabu[c][d] != iteration && 
                                 path_length + D[b, d] < ref_struct_cost
                    ref_struct_cost = path_length + D[b, d]
                    best_c = c
                end
                c = d                                  # Next value for c and d
            end
            if ref_struct_cost < cost    # Admissible reference structure found
                path_modified = true
                c, d = best_c, succ[best_c]        # Update reference structure
                tabu[c][d] =tabu[d][c] =iteration # Don't remove again edge c-d
                path_length += (D[b, d] - D[c, d])
                i, si, succ[b] = b, succ[b], d            # Reverse path b -> c
                while i != c
                    succ[si], i, si = i, si, succ[si]
                end
                b = c
                if path_length + D[a, b] < cost    # A better solution is found
                    cost = path_length + D[a, b]
                    succ[a], last_a = b, b
                    improved = true
                    tour = tsp_succ_to_tour(succ)
                end
            end
        end
        succ = tsp_tour_to_succ(tour)
        a = succ[a]
    end
    return tour, cost
end
