function tsp_SA(d,                       # Distance matrix, must be symmetrical
                tour,                    # Sequence of cities of TSP tour
                tour_length,             # Length of the tour
                initial_temperature,     # SA basic parameters; for instance:
                final_temperature,       # tour_length/n, tour_length/n/n
                alpha)                   # and 0.99
    n = length(tour)
    best_length = tour_length
    best_tour = copy(tour)
    T = initial_temperature
    iteration = 0
    while T > final_temperature
        i = unif(1, n)                   # First city of a move randomly chosen
        j = mod1(i + unif(2, n-2),n)   # Second city is unif successors further
        if j < i i, j = j, i end                # j must be further on the tour
         
        delta = d[tour[i], tour[j]] + d[tour[i + 1], tour[mod1(j + 1, n)]] -
                d[tour[i], tour[i + 1]] - d[tour[j], tour[mod1(j + 1, n)]]
        if delta < 0 || exp(-delta / T) > rando()               # Move accepted
            tour_length += delta  
            for k in 0:((j - i) ÷ 2 - 1)     # Reverse sub-path between i and j
                tour[k + i + 1], tour[j - k] = tour[j - k], tour[k + i + 1]
            end
        end
        if best_length > tour_length             # Store improved best solution
            best_length = tour_length
            best_tour = copy(tour)
            println("SA iteration: $iteration, length: $tour_length")
        end
        
        iteration += 1
        if iteration % (n * n) == 0
            T *= alpha              # Temperature decreases every n*n iteration
        end
    end    
    return best_tour, best_length
end # tsp_SA
