######### Taboo Search for the TSP, based on 2-opt moves
function tsp_TS(d,                                            # Distance matrix
                tour,                                   # Intital tour provided
                tour_length,                           # Length of initial tour
                iterations,                  # Number of tabu search iterations
                min_tabu_duration,                      # Minimal tabu duration
                max_tabu_duration, 
                F)           # Factor for penalizing moves repeatedly performed
    n = length(tour)
    tabu = zeros(Int, n, n)                                         # Tabu list
    count = zeros(Int, n, n)                                       # Move count
    best_tour = copy(tour)
    best_length = tour_length
    for iteration in 1:iterations
        delta_penalty = Inf
        ir, jr = -1, -1                 # Cities retained for performing a move
        # Find best move allowed or aspired
        for i in 1:n-2
            j = i + 2
            while j < n + (i > 1 ? 1 : 0)
                delta =  d[tour[i], tour[j]] + d[tour[i+1], tour[mod1(j+1,n)]]-
                    d[tour[i], tour[i+1]] - d[tour[j], tour[mod1(j+1, n)]]
                penalty = F * (count[tour[i], tour[j]] + 
                               count[tour[i+1], tour[mod1(j+1, n)]])
                # Conditions for accepting a candidate move
                better = delta + penalty < delta_penalty
                allowed = tabu[tour[i], tour[j]] <= iteration || 
                          tabu[tour[i+1], tour[mod1(j+1, n)]] <= iteration
                aspirated = tour_length + delta < best_length
                if better && (allowed || aspirated)
                    delta_penalty, ir, jr = delta + penalty, i, j
                end
                j += 1                                         # Next neighbour
            end
        end

        # Perform retained move
        if delta_penalty < Inf
            tabu[tour[ir], tour[ir+1]] = tabu[tour[jr], tour[mod1(jr+1, n)]] = 
            tabu[tour[ir+1], tour[ir]] = tabu[tour[mod1(jr+1, n)], tour[jr]] = 
            unif(min_tabu_duration, max_tabu_duration) + iteration

            count[tour[ir], tour[ir+1]] += 1
            count[tour[jr], tour[mod1(jr+1, n)]] += 1
            count[tour[ir+1], tour[ir]] += 1
            count[tour[mod1(jr+1, n)], tour[jr]] += 1

            tour_length += d[tour[ir], tour[jr]] + 
              d[tour[ir+1], tour[mod1(jr+1, n)]] - d[tour[ir], tour[ir+1]] - 
              d[tour[jr], tour[mod1(jr+1, n)]]

            for k in 0:div(jr-ir, 2)-1
                tour[k + ir + 1], tour[jr - k] = tour[jr - k], tour[k + ir + 1]
            end
        else
            println("All moves are forbidden, tabu list too long")
        end
        if best_length > tour_length                 # Is there an improvement?
            best_length, best_tour = tour_length, copy(tour)
            println("TS $iteration $best_length")
        end
    end
    return best_tour, best_length
end
