function tsp_noising(d,                                       # Distance matrix
                     tour,                                # TSP tour (sequence)
                     tour_length,                                 # Tour length
                     initial_noise,                                # Parameters
                     final_noise, 
                     alpha)            
    n = length(tour)
    t = build_2opt_data_structure(tour)
    noise = initial_noise
    best_length = tour_length
    iteration = 0

    while noise > final_noise
        i = last_i = unif(2, 2n+1)       # First city of a move randomly chosen
        while t[t[i]] != last_i && t[i] != last_i      # All moves with i tried
            j = t[t[i]]
            while j != last_i && (t[j] != last_i || i != last_i)
                delta = d[i ÷ 2, j ÷ 2] + d[t[i] ÷ 2, t[j] ÷ 2] - 
                        d[i ÷ 2, t[i] ÷ 2] - d[j ÷ 2, t[j] ÷ 2]
                if delta + noise * log(rando()) < 0              # SA criterion
                    tour_length += delta                        # Move accepted
                    best_i, best_j = t[i], t[j]
                    t[i], t[j] = j ⊻ 1, i ⊻ 1 # New successors and predecessors
                    t[best_i ⊻ 1], t[best_j ⊻ 1] = best_j, best_i

                    i = t[i]         # Avoid immediate reverse a degrading move
                    j = t[i]
                    
                    if best_length > tour_length          # New best tour found
                        best_length = tour_length
                        tour = tsp_2opt_data_structure_to_tour(t)
                        println("Noising $iteration $tour_length")
                    end
                end
                iteration += 1
                if iteration % (n*n) == 0  # Decrease noise every n*n iteration
                    noise *= alpha                     
                end
                j = t[j]  
            end # Next j
            i = t[i]  
        end # Next i
    end # while noise > final_noise
    return tour, best_length
end # tsp_noising
