include("tsp_utilities.jl")                                         # Code %*\ref{lst:tsp_utilities}*)

######### Path relinking for the TSP, based on 3-opt neighbourhood
function tsp_path_relinking(d,                                # Distance matrix
                            target,              # Target solution (successors)
                            tour_length,           # Length of current solution
                            succ)                           # Starting solution
    best_succ, best_length = copy(succ), tour_length
    pred = tsp_succ_to_pred(succ)
    best_delta = -1
    while best_delta < Inf
        best_delta = Inf
        i = best_i = best_j = best_k = pred[1]
        while best_delta >= 0 && i != 1
            i = succ[i]
            if succ[i] != target[i]
                j = pred[target[i]]
                k = target[i]
                while k != i
                    if succ[k] != target[k]
                        delta = d[i, succ[j]] + d[j, succ[k]] + d[k, succ[i]] - 
                                d[i, succ[i]] - d[j, succ[j]] - d[k, succ[k]]
                        if delta < best_delta
                            best_delta, best_i, best_j, best_k = delta, i, j, k
                        end
                    end
                    k = succ[k]
                end
            end
        end
        if best_delta < Inf
            i, j, k = best_i, best_j, best_k
            tour_length += best_delta
            pred[succ[i]], pred[succ[j]], pred[succ[k]] = k, i, j
            succ[j], succ[k], succ[i] = succ[k], succ[i], target[i]
            if tour_length < best_length
                best_length, best_succ = tour_length, copy(succ)
             end
        end
    end
    return best_succ, best_length
end
