# Compute the length of a TSP tour
function tsp_length(d, tour)
    n = length(tour)
    return sum(d[tour[i], tour[mod1(i+1,n)]] for i = 1:n)
end

# Build solution representation by successors
function tsp_tour_to_succ(tour)
    n = length(tour)
    succ = Vector{Int}(undef, n)
    for i in 1:n
        succ[tour[i]] = tour[mod1(i + 1, n)]
    end
    return succ
end

# Build solution representation by predecessors
function tsp_succ_to_pred(succ)
    n = length(succ)
    pred = Vector{Int}(undef, n)
    for i in 1:n
        pred[succ[i]] = i
    end
    return pred
end

# Convert solution from successor of each city to city sequence
function tsp_succ_to_tour(succ)
    n = length(succ)
    tour = Vector{Int}(undef, n)
    j = 1
    for i in 1:n
        tour[i] = j
        j = succ[j]
    end
    return tour
end

# Convert a solution given by 2-opt data structure to a standard tour
function tsp_2opt_data_structure_to_tour(t)
    n = length(t) ÷ 2
    tour = Vector{Int}(undef, n)
    j = t[2]
    for i in 1:n
        tour[i] = j ÷ 2
        j = t[j]
    end
    return tour
end

# Compare 2 directed tours; returns the number of different arcs
function tsp_compare(succ_a, succ_b)
    count = 0
    for i = eachindex(succ_a)
        count += ifelse(succ_a[i] != succ_b[i], 1, 0)
    end
    return count
end
