include("random_generators.jl")                                     # Code %*\ref{lst:random_generators}*)
include("tsp_LK.jl")                                                # Code %*\ref{lst:tsp_LK}*)
include("generate_solution_trail.jl")                               # Code %*\ref{lst:generate_solution_trail}*)
include("init_update_trail.jl")                                     # Code %*\ref{lst:init_update_trail}*)

######### Fast Ant System for the TSP
function tsp_FANT(d,                                          # Distance matrix
                  exploitation,         # FANT Parameters: global reinforcement
                  iterations)                  # number of solution to generate
    n = size(d, 1)
    best_cost = typemax(Int)
    exploration = 1
    trail = init_trail(exploration, Matrix{Int}(undef, n, n))
    tour = rand_permutation(n)
    best_sol = copy(tour)
    for i in 1:iterations
        # build solution
        tour, cost = generate_solution_trail(d, tour, trail)
        # improve built solution with a local search
        tour, cost = tsp_LK(d, tour, cost)
        if cost < best_cost
            best_cost = cost
            println("FANT $(i) $(cost)")
            best_sol = copy(tour)
            exploration = 1                 # Reset exploration to lowest value
            trail = init_trail(exploration, trail)
        else
            # Pheromone trace reinforcement - increase memory
            trail, exploration = 
                update_trail(tour, best_sol, exploration, exploitation, trail)
        end
    end
    return best_sol, best_cost
end
